MODULE MultiSkel;

(*
 *	TransSkel multiple window demonstration: Main Module.
 *	
 *	This module performs setup and termination operations, installs the window
 *	and menu handlers, and processes menu item selections.
 *	
 *	There are four window handlers in this demonstration.  The code for each
 *	handler is in its own module.
 *	
 *		Help Window		Scrollable non-editable text window
 *		Edit Window		Non-scrollable editable text window
 *		Zoom Window		Non-manipulable graphics display window
 *		Region Window	Manipulable graphics display window
 *	
 *	 Paul DuBois, 24 June 1986
 *		Modula-2 version  Dennis Cohen, 14 April 1987
 *)

FROM SYSTEM IMPORT ADDRESS;
FROM QuickDraw IMPORT RgnHandle;
FROM MenuManager IMPORT MenuHandle, GetMenu;
FROM WindowManager IMPORT WindowPeek, WindowPtr, FrontWindow, HideWindow,
													ShowWindow, SelectWindow;
FROM DeskManager IMPORT SystemEdit, CloseDeskAcc;
FROM DialogManager IMPORT Alert;
FROM TransSkel IMPORT SkelInit, SkelMain, SkelClobber, SkelApple, SkelMenu,
											SkelWhoa, MClobberProc;
FROM MultiSkelEdit IMPORT EditWindEditMenu, EditWindInit;
FROM MultiSkelRgn IMPORT RgnInit;
FROM MultiSkelZoom IMPORT ZoomInit;
FROM MultiSkelHelp IMPORT HelpWindInit;
FROM MultiSkelGlobals IMPORT editMenu, editWind, fileMenuRes, editMenuRes,
															rgnWind, zoomWind, helpWind, aboutAlrt;

CONST
	open = 1;
	close = 2;
	quit = 4;
	
VAR
	fileMenu: MenuHandle;
	oldClip:	RgnHandle;


(*
 *	Handle selection of About MultiSkel item from the Apple menu
 *)

PROCEDURE DoAbout;

	VAR
		whoCares: INTEGER;

BEGIN
	whoCares := Alert(aboutAlrt, VAL(ADDRESS, NIL));
END DoAbout;


(*
 *	Process selections from the File menu.
 *	
 *		Open		Make all four windows visible.
 *		Close		Hide the frontmost window.  If it belongs to a DA, close the DA.
 *		Quit		Request a halt by calling SkelWhoa.  This makes SkelMain return.
 *)

PROCEDURE DoFile(item: INTEGER);

	VAR
		wPeek:	WindowPeek;
	
	PROCEDURE MyShowWindow(wind: WindowPeek);
	
	(*
		Show a window if it's not visible.  Select the window first, then show it
		so that it comes up in front.  Otherwise it will be drawn in back and then
		brought to the front, which is ugly.
		
		The test for visibility must be done carefully; the WindowManager stores
		255 for true and 0 for false - not a real boolean.
	*)
	
	BEGIN
		IF VAL(CARDINAL,wind^.visible) = 0 THEN
			SelectWindow(VAL(WindowPtr, wind));
			ShowWindow(VAL(WindowPtr, wind));
		END;
	END MyShowWindow;
		
BEGIN
	CASE item OF
		open:	MyShowWindow(VAL(WindowPeek, rgnWind));
					MyShowWindow(VAL(WindowPeek, zoomWind));
					MyShowWindow(VAL(WindowPeek, editWind));
					MyShowWindow(VAL(WindowPeek, helpWind));
		|
		(* Close the front window.  Take into account whether it belongs to a DA *)
		close:	wPeek := VAL(WindowPeek, FrontWindow());
						IF wPeek # NIL THEN
							IF wPeek^.windowKind < 0 THEN CloseDeskAcc(wPeek^.windowKind);
							ELSE HideWindow(FrontWindow());
							END;
						END;
		|
		quit:	SkelWhoa;
		ELSE (* Do nothing *) ;
	END;
END DoFile;


(*
	Process item selected from Edit menu.  First check whether it should get
	routed to a DA.  If not, then route the selection to the text editing window,
	as that is the only one for this application to which edit commands apply.
	(The test for FrontWindow is not strictly necessary, as the Edit menu is
	disabled when any of the other windows is frontmost and so this procedure
	shouldn't get called.)
*)

PROCEDURE DoEdit(item: INTEGER);

BEGIN
	IF NOT SystemEdit(item-1) THEN
		IF FrontWindow() = editWind THEN EditWindEditMenu(item); END;
	END;
END DoEdit;

(*
 *	Initialize menus.  Tell TransSkel to process the Apple Menu automatically,
 *	and associate the proper procedures with the File and Edit Menus.
 *)

PROCEDURE SetUpMenus;

BEGIN
	SkelApple("About MultiSkel", DoAbout);
	fileMenu := GetMenu(fileMenuRes);
	editMenu := GetMenu(editMenuRes);
	SkelMenu(fileMenu, DoFile, VAL(MClobberProc, NIL));
	SkelMenu(editMenu, DoEdit, VAL(MClobberProc, NIL));
END SetUpMenus;


BEGIN
	SkelInit;
	SetUpMenus;
	RgnInit;
	ZoomInit;
	EditWindInit;
	HelpWindInit;
	SkelMain;
	SkelClobber;
END MultiSkel.
